/**************************************************************************
 *  rss.c
    Modified from status.c
 *************************************************************************/
/* CHANGE THESE! */
#define SITE "My Nagios Site"
#define BASE "https://nagios/nagios"
/* End of change */

#include "../common/config.h"
#include "../common/locations.h"
#include "../common/common.h"
#include "../common/objects.h"
#include "../common/comments.h"
#include "../common/statusdata.h"

#include "cgiutils.h"
#include "getcgi.h"
#include "auth.h"
#include "edata.h"


extern int             refresh_rate;
extern time_t          program_start;

extern char main_config_file[MAX_FILENAME_LENGTH];
extern char url_html_path[MAX_FILENAME_LENGTH];
extern char url_docs_path[MAX_FILENAME_LENGTH];
extern char url_images_path[MAX_FILENAME_LENGTH];
extern char url_stylesheets_path[MAX_FILENAME_LENGTH];
extern char url_logo_images_path[MAX_FILENAME_LENGTH];
extern char url_media_path[MAX_FILENAME_LENGTH];
extern char log_file[MAX_FILENAME_LENGTH];

extern hostgroup *hostgroup_list;
extern hoststatus *hoststatus_list;
extern servicestatus *servicestatus_list;
extern service *service_list;
extern host *host_list;


#define MAX_MESSAGE_BUFFER		4096

#define DISPLAY_HOSTS			0
#define DISPLAY_HOSTGROUPS		1

#define STYLE_OVERVIEW			0
#define STYLE_DETAIL			1
#define STYLE_SUMMARY			2
#define STYLE_GRID                      3
#define STYLE_HOST_DETAIL               4

/* HOSTSORT structure */
typedef struct hostsort_struct{
	hoststatus *hststatus;
	struct hostsort_struct *next;
        }hostsort;

/* SERVICESORT structure */
typedef struct servicesort_struct{
	servicestatus *svcstatus;
	struct servicesort_struct *next;
        }servicesort;

hostsort *hostsort_list=NULL;
servicesort *servicesort_list=NULL;

int sort_services(int,int);						/* sorts services */
int sort_hosts(int,int);                                                /* sorts hosts */
int compare_servicesort_entries(int,int,servicesort *,servicesort *);	/* compares service sort entries */
int compare_hostsort_entries(int,int,hostsort *,hostsort *);            /* compares host sort entries */
void free_servicesort_list(void);
void free_hostsort_list(void);

void show_host_status_totals(void);
void show_service_status_totals(void);
void show_service_detail(void);

void show_filters(void);

int passes_host_properties_filter(hoststatus *);
int passes_service_properties_filter(servicestatus *);

void document_header(int);
void document_footer(void);
int process_cgivars(void);


authdata current_authdata;
time_t current_time;

char alert_message[MAX_MESSAGE_BUFFER];
char *host_name=NULL;
char *hostgroup_name=NULL;
int host_alert=FALSE;
int show_all_hosts=TRUE;
int show_all_hostgroups=TRUE;
int display_type=DISPLAY_HOSTS;
int overview_columns=3;
int max_grid_width=8;
int hostgroup_style_type=STYLE_OVERVIEW;

int service_status_types=SERVICE_UNKNOWN|SERVICE_WARNING|SERVICE_CRITICAL|SERVICE_HOST_DOWN|SERVICE_UNREACHABLE;
int all_service_status_types=SERVICE_PENDING|SERVICE_RECOVERY|SERVICE_OK|SERVICE_UNKNOWN|SERVICE_WARNING|SERVICE_CRITICAL|SERVICE_HOST_DOWN|SERVICE_UNREACHABLE;

int host_status_types=HOST_UP|HOST_DOWN|HOST_UNREACHABLE;
int all_host_status_types=HOST_PENDING|HOST_UP|HOST_DOWN|HOST_UNREACHABLE;

int all_service_problems=SERVICE_UNKNOWN|SERVICE_WARNING|SERVICE_CRITICAL|SERVICE_HOST_DOWN|SERVICE_UNREACHABLE;
int all_host_problems=HOST_DOWN|HOST_UNREACHABLE;

/*
unsigned long host_properties=HOST_NOTIFICATIONS_ENABLED|HOST_NO_SCHEDULED_DOWNTIME;
unsigned long service_properties=SERVICE_NOTIFICATIONS_ENABLED|SERVICE_NO_SCHEDULED_DOWNTIME;
*/
unsigned long host_properties=HOST_NOTIFICATIONS_ENABLED;
unsigned long service_properties=SERVICE_NOTIFICATIONS_ENABLED;




int sort_type=SORT_ASCENDING;
int sort_option=SORT_HOSTNAME;

int problem_hosts_down=0;
int problem_hosts_unreachable=0;
int problem_services_critical=0;
int problem_services_warning=0;
int problem_services_unknown=0;

int embedded=FALSE;
int display_header=FALSE;



int main(void){
	int result=OK;
	char *sound=NULL;
	
	time(&current_time);

	/* get the arguments passed in the URL */
	process_cgivars();

	/* reset internal variables */
	reset_cgi_vars();

	/* read the CGI configuration file */
	result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
	if(result==ERROR){
		document_header(FALSE);
		cgi_config_file_error(DEFAULT_CGI_CONFIG_FILE);
		document_footer();
		return ERROR;
	        }

	/* read the main configuration file */
	result=read_main_config_file(main_config_file);
	if(result==ERROR){
		document_header(FALSE);
		main_config_file_error(main_config_file);
		document_footer();
		return ERROR;
	        }

	/* read all object configuration data */
	result=read_all_object_configuration_data(main_config_file,READ_ALL_OBJECT_DATA);
	if(result==ERROR){
		document_header(FALSE);
		object_data_error();
		document_footer();
		return ERROR;
                }

	/* read all status data */
	result=read_all_status_data(DEFAULT_CGI_CONFIG_FILE,READ_ALL_STATUS_DATA);
	if(result==ERROR){
		document_header(FALSE);
		status_data_error();
		document_footer();
		free_memory();
		return ERROR;
                }

	document_header(TRUE);

	/* read in all host and service comments */
	read_comment_data(DEFAULT_CGI_CONFIG_FILE);

	/* read in extended host information */
	read_extended_object_config_data(DEFAULT_CGI_CONFIG_FILE,READ_ALL_EXTENDED_DATA);

	/* get authentication information */
	get_authentication_information(&current_authdata);


	/* bottom portion of screen - service or hostgroup detail */
	show_service_detail();

	document_footer();

	/* free all allocated memory */
	free_memory();
	free_extended_data();
	free_comment_data();

	/* free memory allocated to the sort lists */
	free_servicesort_list();
	free_hostsort_list();

	return OK;
        }


void document_header(int use_stylesheet){
	char date_time[MAX_DATETIME_LENGTH];
	time_t expire_time;

	printf("Cache-Control: no-store\n");
	printf("Pragma: no-cache\n");

	get_time_string(&current_time,date_time,(int)sizeof(date_time),HTTP_DATE_TIME);
	printf("Last-Modified: %s\n",date_time);

	expire_time=(time_t)0L;
	get_time_string(&expire_time,date_time,(int)sizeof(date_time),HTTP_DATE_TIME);
	printf("Expires: %s\n",date_time);

	printf("Content-type: text/xml\n\n");

	printf("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	printf("<?xml-stylesheet title=\"XSL_formatting\" type=\"text/xsl\" href=\"/nagios/rss/rss.xsl\"?>\n");

	printf("<rss version=\"2.0\" xmlns:blogChannel=\"http://backend.userland.com/blogChannelModule\">\n");

	printf("<channel>\n");
	printf("<title>%s</title>\n",SITE);
	printf("<link>%s</link>\n",BASE);
	printf("<description>%s Nagios systems monitoring</description>\n",SITE);
	printf("<copyright>Copyright %s</copyright>\n",SITE);
	printf("<image><title>Nagios: %s</title>\n",SITE);
	printf("  <url>%s/images/sblogo.jpg</url>\n",BASE);
	printf("  <link>%s/</link>\n",BASE);
	printf("</image>\n");

	return;
        }


void document_footer(void){

	printf("</channel></rss>\n");
	return;
        }


int process_cgivars(void){
	char **variables;
	int error=FALSE;
	int x;

	variables=getcgivars();

	for(x=0;variables[x]!=NULL;x++){

		/* do some basic length checking on the variable identifier to prevent buffer overflows */
		if(strlen(variables[x])>=MAX_INPUT_BUFFER-1){
			x++;
			continue;
		        }

		/* we found the service status type argument */
		else if(!strcmp(variables[x],"servicestatustypes")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			service_status_types=atoi(variables[x]);
		        }

		/* we found the host status type argument */
		else if(!strcmp(variables[x],"hoststatustypes")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			host_status_types=atoi(variables[x]);
		        }

		/* we found the service properties argument */
		else if(!strcmp(variables[x],"serviceprops")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			service_properties=strtoul(variables[x],NULL,10);
		        }

		/* we found the host properties argument */
		else if(!strcmp(variables[x],"hostprops")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
		        }

			host_properties=strtoul(variables[x],NULL,10);
			}
	       } /* for */

	/* free memory allocated to the CGI variables */
	free_cgivars(variables);

	return error;
        }



/* display a detailed listing of the status of all services... */
void show_service_detail(void){
	time_t t;
	char date_time[MAX_DATETIME_LENGTH];
	char rss_date_time[MAX_DATETIME_LENGTH];
	char state_duration[48];
	char status[MAX_INPUT_BUFFER];
	char temp_buffer[MAX_INPUT_BUFFER];
	char temp_url[MAX_INPUT_BUFFER];
	char *status_class="";
	char *status_bg_class="";
	char *host_status_bg_class="";
	char *last_host="";
	int new_host=FALSE;
	servicestatus *temp_status=NULL;
	hostgroup *temp_hostgroup=NULL;
	hostextinfo *temp_hostextinfo=NULL;
	serviceextinfo *temp_serviceextinfo=NULL;
	hoststatus *temp_hoststatus=NULL;
	host *temp_host=NULL;
	service *temp_service=NULL;
	int odd=0;
	int total_comments=0;
	int user_has_seen_something=FALSE;
	servicesort *temp_servicesort=NULL;
	int use_sort=FALSE;
	int result=OK;
	int first_entry=TRUE;
	int days;
	int hours;
	int minutes;
	int seconds;
	int duration_error=FALSE;
	int total_entries=0;


	/* sort the service list if necessary */
	if(sort_type!=SORT_NONE){
		result=sort_services(sort_type,sort_option);
		if(result==ERROR)
			use_sort=FALSE;
		else
			use_sort=TRUE;
	        }
	else
		use_sort=FALSE;


	snprintf(temp_url,sizeof(temp_url)-1,"%s/cgi-bin/%s?",BASE,STATUS_CGI);
	temp_url[sizeof(temp_url)-1]='\x0';
	snprintf(temp_buffer,sizeof(temp_buffer)-1,"host=%s",host_name);
	temp_buffer[sizeof(temp_buffer)-1]='\x0';
	strncat(temp_url,temp_buffer,sizeof(temp_url)-strlen(temp_url)-1);
	temp_url[sizeof(temp_url)-1]='\x0';
	if(service_status_types!=all_service_status_types){
		snprintf(temp_buffer,sizeof(temp_buffer)-1,"&servicestatustypes=%d",service_status_types);
		temp_buffer[sizeof(temp_buffer)-1]='\x0';
		strncat(temp_url,temp_buffer,sizeof(temp_url)-strlen(temp_url)-1);
		temp_url[sizeof(temp_url)-1]='\x0';
	        }
	if(host_status_types!=all_host_status_types){
		snprintf(temp_buffer,sizeof(temp_buffer)-1,"&hoststatustypes=%d",host_status_types);
		temp_buffer[sizeof(temp_buffer)-1]='\x0';
		strncat(temp_url,temp_buffer,sizeof(temp_url)-strlen(temp_url)-1);
		temp_url[sizeof(temp_url)-1]='\x0';
	        }
	if(service_properties!=0){
		snprintf(temp_buffer,sizeof(temp_buffer)-1,"&serviceprops=%lu",service_properties);
		temp_buffer[sizeof(temp_buffer)-1]='\x0';
		strncat(temp_url,temp_buffer,sizeof(temp_url)-strlen(temp_url)-1);
		temp_url[sizeof(temp_url)-1]='\x0';
	        }
	if(host_properties!=0){
		snprintf(temp_buffer,sizeof(temp_buffer)-1,"&hostprops=%lu",host_properties);
		temp_buffer[sizeof(temp_buffer)-1]='\x0';
		strncat(temp_url,temp_buffer,sizeof(temp_url)-strlen(temp_url)-1);
		temp_url[sizeof(temp_url)-1]='\x0';
	        }

	/* the main list of services */
	/* check all services... */
	while(1){

		/* get the next service to display */
		if(use_sort==TRUE){
			if(first_entry==TRUE)
				temp_servicesort=servicesort_list;
			else
				temp_servicesort=temp_servicesort->next;
			if(temp_servicesort==NULL)
				break;
			temp_status=temp_servicesort->svcstatus;
	                }
		else{
			if(first_entry==TRUE)
				temp_status=servicestatus_list;
			else
				temp_status=temp_status->next;
		        }

		if(temp_status==NULL)
			break;

		first_entry=FALSE;

		/* find the service  */
		temp_service=find_service(temp_status->host_name,temp_status->description,NULL);

		/* if we couldn't find the service, go to the next service */
		if(temp_service==NULL)
			continue;

		/* find the host */
		temp_host=find_host(temp_service->host_name,NULL);

		/* make sure user has rights to see this... */
		if(is_authorized_for_service(temp_service,&current_authdata)==FALSE)
			continue;

		user_has_seen_something=TRUE;

		/* get the host status information */
		temp_hoststatus=find_hoststatus(temp_service->host_name);

		/* see if we should display services for hosts with tis type of status */
		if(!(host_status_types & temp_hoststatus->status))
			continue;

		/* see if we should display this type of service status */
		if(!(service_status_types & temp_status->status))
			continue;	

		/* check host properties filter */
		if(passes_host_properties_filter(temp_hoststatus)==FALSE)
			continue;

		/* check service properties filter */
		if(passes_service_properties_filter(temp_status)==FALSE)
			continue;



		if(strcmp(last_host,temp_status->host_name))
			new_host=TRUE;
		else
			new_host=FALSE;

		/* SJS June 2004
		   If not a new host, and the host is DOWN or UNREACHABLE, then dont bother to list 
		   all of the other services.  Just continue here. */
		if( new_host==FALSE && 
			(temp_hoststatus->status==HOST_DOWN || temp_hoststatus->status==HOST_UNREACHABLE)){
			total_entries++; 
			continue;
		}

		/* SJS June 2004 End */

		if(odd)
			odd=0;
		else
			odd=1;

		/* keep track of total number of services we're displaying */
		total_entries++;

	        /* get the last state change time */
		t=temp_status->last_state_change;
		if(!t) { t = program_start; }
		get_time_string(&t,rss_date_time,(int)sizeof(rss_date_time),HTTP_DATE_TIME);
		get_time_string(&t,date_time,(int)sizeof(date_time),SHORT_DATE_TIME);
		if((unsigned long)temp_status->last_check==0L)
			strcpy(date_time,"N/A");

		if(temp_status->status==SERVICE_PENDING){
			strncpy(status,"PENDING",sizeof(status));
	                }
		else if(temp_status->status==SERVICE_OK){
			strncpy(status,"OK",sizeof(status));
	                }
		else if(temp_status->status==SERVICE_RECOVERY){
			strncpy(status,"RECOVERED",sizeof(status));
	                }
		else if(temp_status->status==SERVICE_WARNING){
			strncpy(status,"WARNING",sizeof(status));
	                }
		else if(temp_status->status==SERVICE_UNKNOWN){
			strncpy(status,"UNKNOWN",sizeof(status));
	                }
		else if(temp_status->status==SERVICE_CRITICAL){
			strncpy(status,"CRITICAL",sizeof(status));
	                }
		else if(temp_status->status==SERVICE_HOST_DOWN){
			strncpy(status,"HOST DOWN",sizeof(status));
	                }
		else if(temp_status->status==SERVICE_UNREACHABLE){
			strncpy(status,"UNREACHABLE",sizeof(status));
	                }
		status[sizeof(status)-1]='\x0';

		/* host name column */

		/* find extended information for this host */
		temp_hostextinfo=find_hostextinfo(temp_status->host_name);
		total_comments=number_of_host_comments(temp_host->name);

		/* SJS June 2004
		  If the host is DOWN or UNREACHABLE, then all its services will of course
		  be down.  So, don't display them: Just give the host info and skip.  */
		if(( temp_hoststatus->status==HOST_DOWN || temp_hoststatus->status==HOST_UNREACHABLE )){
			t=0;
			duration_error=FALSE;
			if(temp_hoststatus->last_state_change==(time_t)0){
				if(program_start>current_time)
					duration_error=TRUE;
				else
					t=current_time-program_start;
			} else {
				if(temp_hoststatus->last_state_change>current_time)
					duration_error=TRUE;
				else
					t=current_time-temp_hoststatus->last_state_change;
			}
			get_time_breakdown((unsigned long)t,&days,&hours,&minutes,&seconds);
			if(duration_error==TRUE)
				snprintf(state_duration,sizeof(state_duration)-1,"???");
			else
				snprintf(state_duration,sizeof(state_duration)-1,"%2dd %2dh %2dm %2ds%s",days,hours,minutes,seconds,(temp_hoststatus->last_state_change==(time_t)0)?"+":"");
			state_duration[sizeof(state_duration)-1]='\x0';
			t=temp_hoststatus->last_state_change;
			if(!t) { t = program_start; }
			get_time_string(&t,rss_date_time,(int)sizeof(rss_date_time),HTTP_DATE_TIME);
			get_time_string(&t,date_time,(int)sizeof(date_time),SHORT_DATE_TIME);
			if((unsigned long)temp_hoststatus->last_check==0L)
				strcpy(date_time,"");

			printf("<item><title>");
			if(temp_hoststatus->scheduled_downtime_depth>0) { printf("(SCHED) "); }
			printf("%s: ",temp_status->host_name);
			if( temp_hoststatus->status==HOST_UNREACHABLE )  {
				printf("UNREACHABLE");
			} else {
				printf("DOWN");
			}
			printf("</title>\n<link>%s/cgi-bin/%s?type=%d&amp;host=%s</link>\n",
				BASE,EXTINFO_CGI,DISPLAY_HOST_INFO,url_encode(temp_status->host_name));
			printf("<description><h4>%s </h4>Host %s is %s <br />%s<br /><br />\n", 
				temp_host->alias,
				temp_status->host_name,
				((temp_hoststatus->status==HOST_UNREACHABLE)?"UNREACHABLE":
					((temp_hoststatus->status==HOST_DOWN)?"DOWN":"PENDING")),
				temp_hoststatus->information);
			if(temp_hoststatus->scheduled_downtime_depth>0) {
				printf("<I>This host is in a period of scheduled downtime.</I> <br />");
			}
			if(temp_hoststatus->notifications_enabled==FALSE) {
				printf("<I>Notifications have been disabled.</I> <br />");
			}
			if(temp_hoststatus->problem_has_been_acknowledged==TRUE) {
				printf("<I>This problem has been acknowledged</I> <br />");
			} else {
				printf("<A href=\"%s/cgi-bin/%s?cmd_typ=%d&amp;host=%s&amp;",
					BASE,COMMAND_CGI,CMD_ACKNOWLEDGE_HOST_PROBLEM,
					url_encode(temp_status->host_name));
				printf("com_author=%s&amp;com_data=%s&amp;cmd_mod=2\">Acknowledge this problem</A> <br />",
					url_encode(current_authdata.username),"Acknowledged+via+RSS+feed");
			}
			printf("Incident occurred at: %s<br />\n",date_time);
/*			printf("Duration: %s<br />\n",state_duration); */
			if((temp_hostextinfo!=NULL) && (temp_hostextinfo->notes_url!=NULL)){
				printf("<A href=\"");
				print_host_notes_url(temp_hostextinfo);
				printf("\">Host extended information</a> <br />");
			}
			printf("</description>\n<pubDate>%s</pubDate>\n",rss_date_time);
			printf("<guid>%s:%d:%s</guid>\n",temp_status->host_name,temp_hoststatus->status,
				date_time); 	
			printf("<author>%s</author>\n",temp_status->host_name);
			printf("<comments>%s</comments>\n",temp_hoststatus->information);
			printf("</item>\n\n");
			last_host=temp_status->host_name;
			continue;
		}
		/* if End */

		if (temp_status->current_attempt < temp_status->max_attempts) { continue; } 

		total_comments=number_of_service_comments(temp_service->host_name,temp_service->description);
		temp_serviceextinfo=find_serviceextinfo(temp_service->host_name,temp_service->description);
		/* service name column */

		/* state duration calculation... */
		t=0;
		duration_error=FALSE;
		if(temp_status->last_state_change==(time_t)0){
			if(program_start>current_time)
				duration_error=TRUE;
			else
				t=current_time-program_start;
		        }
		else{
			if(temp_status->last_state_change>current_time)
				duration_error=TRUE;
			else
				t=current_time-temp_status->last_state_change;
		        }
		get_time_breakdown((unsigned long)t,&days,&hours,&minutes,&seconds);
		if(duration_error==TRUE)
			snprintf(state_duration,sizeof(state_duration)-1,"???");
		else
			snprintf(state_duration,sizeof(state_duration)-1,"%2dd %2dh %2dm %2ds%s",days,hours,minutes,seconds,(temp_status->last_state_change==(time_t)0)?"+":"");
		state_duration[sizeof(state_duration)-1]='\x0';

		printf("<item><title>");
		if(temp_status->scheduled_downtime_depth>0) { printf("(SCHED) "); }
		printf("%s: %s: %s",
			temp_status->host_name,status,temp_status->description
			);
		printf("</title>\n<link>%s/cgi-bin/%s?type=%d&amp;host=%s&amp;",
			BASE,EXTINFO_CGI,DISPLAY_SERVICE_INFO,url_encode(temp_status->host_name));
		printf("service=%s</link>\n",
			url_encode(temp_status->description));
		printf("<description><h4>%s</h4> Host name: %s <br />Service: %s <br />Status: %s <br />%s <br /><br />\n",
			temp_host->alias,temp_status->host_name,temp_status->description,status,temp_status->information);
		if(temp_status->scheduled_downtime_depth>0) {
			printf("<i>This service is in a period of scheduled downtime.</i> <br />");
		} else if(temp_hoststatus->scheduled_downtime_depth>0) {
			printf("<i>This service is on a host in a period of scheduled downtime.</i> <br />");
		}
		if(temp_status->notifications_enabled==FALSE) {
			printf("<i>Notifications have been disabled.</i> <br />");
		}
		if(temp_status->problem_has_been_acknowledged==TRUE) {
			printf("<i>This problem has been acknowledged</i> <br />");
		} else {
			printf("<A href=\"%s/cgi-bin/%s?cmd_typ=%d&amp;host=%s&amp;",
				BASE,COMMAND_CGI,CMD_ACKNOWLEDGE_SVC_PROBLEM,
				url_encode(temp_status->host_name));
			printf("service=%s&amp;",
				url_encode(temp_status->description));
			printf("com_author=%s&amp;com_data=%s&amp;cmd_mod=2\">Acknowledge this problem</A> <br />",
				url_encode(current_authdata.username),"Acknowledged+via+RSS+feed");
		}
		printf("Incident occurred at: %s <br />\n",date_time);
/*		printf("Duration: %s<br />\n", state_duration); */
		if((temp_serviceextinfo!=NULL) && (temp_serviceextinfo->notes_url!=NULL)){
			printf("<A href=\"");
			print_service_notes_url(temp_serviceextinfo);
			printf("\">Service extended information</A>");
		}
		printf("</description>\n<pubDate>%s</pubDate>\n",rss_date_time);
		printf("<guid>%s:%s:%d:%s</guid>\n",temp_status->host_name,temp_status->description,
			temp_status->status, date_time);
		printf("<author>%s</author>\n",temp_status->host_name);
		printf("<comments>%s</comments>\n",temp_status->information);
		printf("</item>\n\n");

		last_host=temp_status->host_name;
	} /* while */
	return;
} /* function */





/******************************************************************/
/**********  SERVICE SORTING & FILTERING FUNCTIONS  ***************/
/******************************************************************/


/* sorts the service list */
int sort_services(int s_type, int s_option){
	servicesort *new_servicesort;
	servicesort *last_servicesort;
	servicesort *temp_servicesort;
	servicestatus *temp_svcstatus;

	if(s_type==SORT_NONE)
		return ERROR;

	if(servicestatus_list==NULL)
		return ERROR;

	/* sort all services status entries */
	for(temp_svcstatus=servicestatus_list;temp_svcstatus!=NULL;temp_svcstatus=temp_svcstatus->next){

		/* allocate memory for a new sort structure */
		new_servicesort=(servicesort *)malloc(sizeof(servicesort));
		if(new_servicesort==NULL)
			return ERROR;

		new_servicesort->svcstatus=temp_svcstatus;

		last_servicesort=servicesort_list;
		for(temp_servicesort=servicesort_list;temp_servicesort!=NULL;temp_servicesort=temp_servicesort->next){

			if(compare_servicesort_entries(s_type,s_option,new_servicesort,temp_servicesort)==TRUE){
				new_servicesort->next=temp_servicesort;
				if(temp_servicesort==servicesort_list)
					servicesort_list=new_servicesort;
				else
					last_servicesort->next=new_servicesort;
				break;
		                }
			else
				last_servicesort=temp_servicesort;
	                }

		if(servicesort_list==NULL){
			new_servicesort->next=NULL;
			servicesort_list=new_servicesort;
	                }
		else if(temp_servicesort==NULL){
			new_servicesort->next=NULL;
			last_servicesort->next=new_servicesort;
	                }
	        }

	return OK;
        }


int compare_servicesort_entries(int s_type, int s_option, servicesort *new_servicesort, servicesort *temp_servicesort){
	servicestatus *new_svcstatus;
	servicestatus *temp_svcstatus;
	time_t nt;
	time_t tt;

	new_svcstatus=new_servicesort->svcstatus;
	temp_svcstatus=temp_servicesort->svcstatus;

	if(s_type==SORT_ASCENDING){

		if(s_option==SORT_LASTCHECKTIME){
			if(new_svcstatus->last_check < temp_svcstatus->last_check)
				return TRUE;
			else
				return FALSE;
		        }
		else if(s_option==SORT_CURRENTATTEMPT){
			if(new_svcstatus->current_attempt < temp_svcstatus->current_attempt)
				return TRUE;
			else
				return FALSE;
		        }
		else if(s_option==SORT_SERVICESTATUS){
			if(new_svcstatus->status < temp_svcstatus->status)
				return TRUE;
/* SJS Addition Nov 05 */
			else if(new_svcstatus->status > temp_svcstatus->status)
				return FALSE;
			else if(strcasecmp(new_svcstatus->host_name,temp_svcstatus->host_name)<0)
				return TRUE;
/* SJS addition end */
			else
				return FALSE;
		        }
		else if(s_option==SORT_HOSTNAME){
			if(strcasecmp(new_svcstatus->host_name,temp_svcstatus->host_name)<0)
				return TRUE;
			else
				return FALSE;
		        }
		else if(s_option==SORT_SERVICENAME){
			if(strcasecmp(new_svcstatus->description,temp_svcstatus->description)<0)
				return TRUE;
			else
				return FALSE;
		        }
		else if(s_option==SORT_STATEDURATION){
			if(new_svcstatus->last_state_change==(time_t)0)
				nt=(program_start>current_time)?0:(current_time-program_start);
			else
				nt=(new_svcstatus->last_state_change>current_time)?0:(current_time-new_svcstatus->last_state_change);
			if(temp_svcstatus->last_state_change==(time_t)0)
				tt=(program_start>current_time)?0:(current_time-program_start);
			else
				tt=(temp_svcstatus->last_state_change>current_time)?0:(current_time-temp_svcstatus->last_state_change);
			if(nt<tt)
				return TRUE;
			else
				return FALSE;
		        }
	        }
	else{
		if(s_option==SORT_LASTCHECKTIME){
			if(new_svcstatus->last_check > temp_svcstatus->last_check)
				return TRUE;
			else
				return FALSE;
		        }
		else if(s_option==SORT_CURRENTATTEMPT){
			if(new_svcstatus->current_attempt > temp_svcstatus->current_attempt)
				return TRUE;
			else
				return FALSE;
		        }
		else if(s_option==SORT_SERVICESTATUS){
			if(new_svcstatus->status > temp_svcstatus->status)
				return TRUE;
			else
				return FALSE;
		        }
		else if(s_option==SORT_HOSTNAME){
			if(strcasecmp(new_svcstatus->host_name,temp_svcstatus->host_name)>0)
				return TRUE;
			else
				return FALSE;
		        }
		else if(s_option==SORT_SERVICENAME){
			if(strcasecmp(new_svcstatus->description,temp_svcstatus->description)>0)
				return TRUE;
			else
				return FALSE;
		        }
		else if(s_option==SORT_STATEDURATION){
			if(new_svcstatus->last_state_change==(time_t)0)
				nt=(program_start>current_time)?0:(current_time-program_start);
			else
				nt=(new_svcstatus->last_state_change>current_time)?0:(current_time-new_svcstatus->last_state_change);
			if(temp_svcstatus->last_state_change==(time_t)0)
				tt=(program_start>current_time)?0:(current_time-program_start);
			else
				tt=(temp_svcstatus->last_state_change>current_time)?0:(current_time-temp_svcstatus->last_state_change);
			if(nt>tt)
				return TRUE;
			else
				return FALSE;
		        }
	        }

	return TRUE;
        }



/* sorts the host list */
int sort_hosts(int s_type, int s_option){
	hostsort *new_hostsort;
	hostsort *last_hostsort;
	hostsort *temp_hostsort;
	hoststatus *temp_hststatus;

	if(s_type==SORT_NONE)
		return ERROR;

	if(hoststatus_list==NULL)
		return ERROR;

	/* sort all hosts status entries */
	for(temp_hststatus=hoststatus_list;temp_hststatus!=NULL;temp_hststatus=temp_hststatus->next){

		/* allocate memory for a new sort structure */
		new_hostsort=(hostsort *)malloc(sizeof(hostsort));
		if(new_hostsort==NULL)
			return ERROR;

		new_hostsort->hststatus=temp_hststatus;

		last_hostsort=hostsort_list;
		for(temp_hostsort=hostsort_list;temp_hostsort!=NULL;temp_hostsort=temp_hostsort->next){

			if(compare_hostsort_entries(s_type,s_option,new_hostsort,temp_hostsort)==TRUE){
				new_hostsort->next=temp_hostsort;
				if(temp_hostsort==hostsort_list)
					hostsort_list=new_hostsort;
				else
					last_hostsort->next=new_hostsort;
				break;
		                }
			else
				last_hostsort=temp_hostsort;
	                }

		if(hostsort_list==NULL){
			new_hostsort->next=NULL;
			hostsort_list=new_hostsort;
	                }
		else if(temp_hostsort==NULL){
			new_hostsort->next=NULL;
			last_hostsort->next=new_hostsort;
	                }
	        }

	return OK;
        }


int compare_hostsort_entries(int s_type, int s_option, hostsort *new_hostsort, hostsort *temp_hostsort){
	hoststatus *new_hststatus;
	hoststatus *temp_hststatus;
	time_t nt;
	time_t tt;

	new_hststatus=new_hostsort->hststatus;
	temp_hststatus=temp_hostsort->hststatus;

	if(s_type==SORT_ASCENDING){

		if(s_option==SORT_LASTCHECKTIME){
			if(new_hststatus->last_check < temp_hststatus->last_check)
				return TRUE;
			else
				return FALSE;
		        }
		else if(s_option==SORT_HOSTSTATUS){
			if(new_hststatus->status <= temp_hststatus->status)
				return TRUE;
			else
				return FALSE;
		        }
		else if(s_option==SORT_HOSTNAME){
			if(strcasecmp(new_hststatus->host_name,temp_hststatus->host_name)<0)
				return TRUE;
			else
				return FALSE;
		        }
		else if(s_option==SORT_STATEDURATION){
			if(new_hststatus->last_state_change==(time_t)0)
				nt=(program_start>current_time)?0:(current_time-program_start);
			else
				nt=(new_hststatus->last_state_change>current_time)?0:(current_time-new_hststatus->last_state_change);
			if(temp_hststatus->last_state_change==(time_t)0)
				tt=(program_start>current_time)?0:(current_time-program_start);
			else
				tt=(temp_hststatus->last_state_change>current_time)?0:(current_time-temp_hststatus->last_state_change);
			if(nt<tt)
				return TRUE;
			else
				return FALSE;
		        }
	        }
	else{
		if(s_option==SORT_LASTCHECKTIME){
			if(new_hststatus->last_check > temp_hststatus->last_check)
				return TRUE;
			else
				return FALSE;
		        }
		else if(s_option==SORT_HOSTSTATUS){
			if(new_hststatus->status > temp_hststatus->status)
				return TRUE;
			else
				return FALSE;
		        }
		else if(s_option==SORT_HOSTNAME){
			if(strcasecmp(new_hststatus->host_name,temp_hststatus->host_name)>0)
				return TRUE;
			else
				return FALSE;
		        }
		else if(s_option==SORT_STATEDURATION){
			if(new_hststatus->last_state_change==(time_t)0)
				nt=(program_start>current_time)?0:(current_time-program_start);
			else
				nt=(new_hststatus->last_state_change>current_time)?0:(current_time-new_hststatus->last_state_change);
			if(temp_hststatus->last_state_change==(time_t)0)
				tt=(program_start>current_time)?0:(current_time-program_start);
			else
				tt=(temp_hststatus->last_state_change>current_time)?0:(current_time-temp_hststatus->last_state_change);
			if(nt>tt)
				return TRUE;
			else
				return FALSE;
		        }
	        }

	return TRUE;
        }



/* free all memory allocated to the servicesort structures */
void free_servicesort_list(void){
	servicesort *this_servicesort;
	servicesort *next_servicesort;

	/* free memory for the servicesort list */
	for(this_servicesort=servicesort_list;this_servicesort!=NULL;this_servicesort=next_servicesort){
		next_servicesort=this_servicesort->next;
		free(this_servicesort);
	        }

	return;
        }


/* free all memory allocated to the hostsort structures */
void free_hostsort_list(void){
	hostsort *this_hostsort;
	hostsort *next_hostsort;

	/* free memory for the hostsort list */
	for(this_hostsort=hostsort_list;this_hostsort!=NULL;this_hostsort=next_hostsort){
		next_hostsort=this_hostsort->next;
		free(this_hostsort);
	        }

	return;
        }



/* check host properties filter */
int passes_host_properties_filter(hoststatus *temp_hoststatus){

	if((host_properties & HOST_SCHEDULED_DOWNTIME) && temp_hoststatus->scheduled_downtime_depth<=0)
		return FALSE;

	if((host_properties & HOST_NO_SCHEDULED_DOWNTIME) && temp_hoststatus->scheduled_downtime_depth>0)
		return FALSE;

	if((host_properties & HOST_STATE_ACKNOWLEDGED) && temp_hoststatus->problem_has_been_acknowledged==FALSE)
		return FALSE;

	if((host_properties & HOST_STATE_UNACKNOWLEDGED) && temp_hoststatus->problem_has_been_acknowledged==TRUE)
		return FALSE;

	if((host_properties & HOST_CHECKS_DISABLED) && temp_hoststatus->checks_enabled==TRUE)
		return FALSE;

	if((host_properties & HOST_CHECKS_ENABLED) && temp_hoststatus->checks_enabled==FALSE)
		return FALSE;

	if((host_properties & HOST_EVENT_HANDLER_DISABLED) && temp_hoststatus->event_handler_enabled==TRUE)
		return FALSE;

	if((host_properties & HOST_EVENT_HANDLER_ENABLED) && temp_hoststatus->event_handler_enabled==FALSE)
		return FALSE;

	if((host_properties & HOST_FLAP_DETECTION_DISABLED) && temp_hoststatus->flap_detection_enabled==TRUE)
		return FALSE;

	if((host_properties & HOST_FLAP_DETECTION_ENABLED) && temp_hoststatus->flap_detection_enabled==FALSE)
		return FALSE;

	if((host_properties & HOST_IS_FLAPPING) && temp_hoststatus->is_flapping==FALSE)
		return FALSE;

	if((host_properties & HOST_IS_NOT_FLAPPING) && temp_hoststatus->is_flapping==TRUE)
		return FALSE;

	if((host_properties & HOST_NOTIFICATIONS_DISABLED) && temp_hoststatus->notifications_enabled==TRUE)
		return FALSE;

	if((host_properties & HOST_NOTIFICATIONS_ENABLED) && temp_hoststatus->notifications_enabled==FALSE)
		return FALSE;

	return TRUE;
        }



/* check service properties filter */
int passes_service_properties_filter(servicestatus *temp_servicestatus){

	if((service_properties & SERVICE_SCHEDULED_DOWNTIME) && temp_servicestatus->scheduled_downtime_depth<=0)
		return FALSE;

	if((service_properties & SERVICE_NO_SCHEDULED_DOWNTIME) && temp_servicestatus->scheduled_downtime_depth>0)
		return FALSE;

	if((service_properties & SERVICE_STATE_ACKNOWLEDGED) && temp_servicestatus->problem_has_been_acknowledged==FALSE)
		return FALSE;

	if((service_properties & SERVICE_STATE_UNACKNOWLEDGED) && temp_servicestatus->problem_has_been_acknowledged==TRUE)
		return FALSE;

	if((service_properties & SERVICE_CHECKS_DISABLED) && temp_servicestatus->checks_enabled==TRUE)
		return FALSE;

	if((service_properties & SERVICE_CHECKS_ENABLED) && temp_servicestatus->checks_enabled==FALSE)
		return FALSE;

	if((service_properties & SERVICE_EVENT_HANDLER_DISABLED) && temp_servicestatus->event_handler_enabled==TRUE)
		return FALSE;

	if((service_properties & SERVICE_EVENT_HANDLER_ENABLED) && temp_servicestatus->event_handler_enabled==FALSE)
		return FALSE;

	if((service_properties & SERVICE_FLAP_DETECTION_DISABLED) && temp_servicestatus->flap_detection_enabled==TRUE)
		return FALSE;

	if((service_properties & SERVICE_FLAP_DETECTION_ENABLED) && temp_servicestatus->flap_detection_enabled==FALSE)
		return FALSE;

	if((service_properties & SERVICE_IS_FLAPPING) && temp_servicestatus->is_flapping==FALSE)
		return FALSE;

	if((service_properties & SERVICE_IS_NOT_FLAPPING) && temp_servicestatus->is_flapping==TRUE)
		return FALSE;

	if((service_properties & SERVICE_NOTIFICATIONS_DISABLED) && temp_servicestatus->notifications_enabled==TRUE)
		return FALSE;

	if((service_properties & SERVICE_NOTIFICATIONS_ENABLED) && temp_servicestatus->notifications_enabled==FALSE)
		return FALSE;

	if((service_properties & SERVICE_PASSIVE_CHECKS_DISABLED) && temp_servicestatus->accept_passive_service_checks==TRUE)
		return FALSE;

	if((service_properties & SERVICE_PASSIVE_CHECKS_ENABLED) && temp_servicestatus->accept_passive_service_checks==FALSE)
		return FALSE;

	if((service_properties & SERVICE_PASSIVE_CHECK) && temp_servicestatus->check_type==SERVICE_CHECK_ACTIVE)
		return FALSE;

	if((service_properties & SERVICE_ACTIVE_CHECK) && temp_servicestatus->check_type==SERVICE_CHECK_PASSIVE)
		return FALSE;

	return TRUE;
        }

